@@ -22,7 +22,7 @@ use Convert::Scalar ':utf8';
 
 use base 'Exporter';
 
-$VERSION = 0.142;
+$VERSION = 0.143;
 @EXPORT = qw();
 
 our $verbose = 1;
@@ -78,7 +78,7 @@ sub export_po {
    my $st = sql_exec \my($id, $lang1, $context, $lang2, $flags, $msg),
                      "select i.id, i.lang, i.context, s.lang, s.flags*1, s.msg
                       from msgid i, msgstr s where i.nr = s.nr and i.domain = ?
-                      order by 4,1",
+                      order by 4,1,2,6",
                      $domain;
 
    while ($st->fetch) {
